@@ -1,6 +1,4 @@
 /*
- * $Id: xftdraw.c,v 1.4 2005/07/03 07:00:57 daniels Exp $
- *
  * Copyright © 2000 Keith Packard
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
@@ -90,7 +88,7 @@ XftDrawDepth (XftDraw *draw)
 	Window		    root;
 	int		    x, y;
 	unsigned int	    width, height, borderWidth, depth;
-	if (XGetGeometry (draw->dpy, draw->drawable, 
+	if (XGetGeometry (draw->dpy, draw->drawable,
 			  &root, &x, &y, &width, &height,
 			  &borderWidth, &depth))
 	    draw->depth = depth;
@@ -106,7 +104,7 @@ XftDrawBitsPerPixel (XftDraw	*draw)
 	XPixmapFormatValues *formats;
 	int		    nformats;
 	unsigned int	    depth;
-	
+
 	if ((depth = XftDrawDepth (draw)) &&
 	    (formats = XListPixmapFormats (draw->dpy, &nformats)))
 	{
@@ -116,7 +114,7 @@ XftDrawBitsPerPixel (XftDraw	*draw)
 	    {
 		if (formats[i].depth == depth)
 		{
-		    draw->bits_per_pixel = formats[i].bits_per_pixel;
+		    draw->bits_per_pixel = (unsigned)formats[i].bits_per_pixel;
 		    break;
 		}
 	    }
@@ -136,8 +134,8 @@ XftDrawCreate (Display   *dpy,
 
     draw = (XftDraw *) malloc (sizeof (XftDraw));
     if (!draw)
-	return 0;
-    
+	return NULL;
+
     draw->dpy = dpy;
     draw->drawable = drawable;
     draw->screen = _XftDrawScreen (dpy, drawable, visual);
@@ -146,7 +144,7 @@ XftDrawCreate (Display   *dpy,
     draw->visual = visual;
     draw->colormap = colormap;
     draw->render.pict = 0;
-    draw->core.gc = 0;
+    draw->core.gc = NULL;
     draw->core.use_pixmap = 0;
     draw->clip_type = XftClipTypeNone;
     draw->subwindow_mode = ClipByChildren;
@@ -162,16 +160,16 @@ XftDrawCreateBitmap (Display	*dpy,
 
     draw = (XftDraw *) malloc (sizeof (XftDraw));
     if (!draw)
-	return 0;
+	return NULL;
     draw->dpy = dpy;
     draw->drawable = (Drawable) bitmap;
-    draw->screen = _XftDrawScreen (dpy, bitmap, 0);
+    draw->screen = _XftDrawScreen (dpy, bitmap, NULL);
     draw->depth = 1;
     draw->bits_per_pixel = 1;
-    draw->visual = 0;
+    draw->visual = NULL;
     draw->colormap = 0;
     draw->render.pict = 0;
-    draw->core.gc = 0;
+    draw->core.gc = NULL;
     draw->core.use_pixmap = 0;
     draw->clip_type = XftClipTypeNone;
     draw->subwindow_mode = ClipByChildren;
@@ -188,16 +186,16 @@ XftDrawCreateAlpha (Display *dpy,
 
     draw = (XftDraw *) malloc (sizeof (XftDraw));
     if (!draw)
-	return 0;
+	return NULL;
     draw->dpy = dpy;
     draw->drawable = (Drawable) pixmap;
-    draw->screen = _XftDrawScreen (dpy, pixmap, 0);
-    draw->depth = depth;
+    draw->screen = _XftDrawScreen (dpy, pixmap, NULL);
+    draw->depth = (unsigned)depth;
     draw->bits_per_pixel = 0;	/* don't find out until we need it */
-    draw->visual = 0;
+    draw->visual = NULL;
     draw->colormap = 0;
     draw->render.pict = 0;
-    draw->core.gc = 0;
+    draw->core.gc = NULL;
     draw->core.use_pixmap = 0;
     draw->clip_type = XftClipTypeNone;
     draw->subwindow_mode = ClipByChildren;
@@ -210,17 +208,17 @@ _XftDrawFormat (XftDraw	*draw)
 {
     XftDisplayInfo  *info = _XftDisplayInfoGet (draw->dpy, True);
 
-    if (!info->hasRender)
-	return 0;
+    if (!info || !info->hasRender)
+	return NULL;
 
-    if (draw->visual == 0)
+    if (draw->visual == NULL)
     {
 	XRenderPictFormat   pf;
 
 	pf.type = PictTypeDirect;
-	pf.depth = XftDrawDepth (draw);
+	pf.depth = (int)XftDrawDepth (draw);
 	pf.direct.alpha = 0;
-	pf.direct.alphaMask = (1 << pf.depth) - 1;
+	pf.direct.alphaMask = (short)((1 << pf.depth) - 1);
 	return XRenderFindFormat (draw->dpy,
 				  (PictFormatType|
 				   PictFormatDepth|
@@ -246,7 +244,7 @@ XftDrawChange (XftDraw	*draw,
     if (draw->core.gc)
     {
 	XFreeGC (draw->dpy, draw->core.gc);
-	draw->core.gc = 0;
+	draw->core.gc = NULL;
     }
 }
 
@@ -303,9 +301,9 @@ XftDrawSrcPicture (XftDraw *draw, _Xconst XftColor *color)
     int		    i;
     XftColor	    bitmapColor;
 
-    if (!info)
+    if (!info || !info->solidFormat)
 	return 0;
-    
+
     /*
      * Monochrome targets require special handling; the PictOp controls
      * the color, and the color must be opaque
@@ -324,9 +322,9 @@ XftDrawSrcPicture (XftDraw *draw, _Xconst XftColor *color)
      */
     for (i = 0; i < XFT_NUM_SOLID_COLOR; i++)
     {
-	if (info->colors[i].pict && 
+	if (info->colors[i].pict &&
 	    info->colors[i].screen == draw->screen &&
-	    !memcmp ((void *) &color->color, 
+	    !memcmp ((void *) &color->color,
 		     (void *) &info->colors[i].color,
 		     sizeof (XRenderColor)))
 	    return info->colors[i].pict;
@@ -335,39 +333,52 @@ XftDrawSrcPicture (XftDraw *draw, _Xconst XftColor *color)
      * Pick one to replace at random
      */
     i = (unsigned int) rand () % XFT_NUM_SOLID_COLOR;
-    /*
-     * Recreate if it was for the wrong screen
-     */
-    if (info->colors[i].screen != draw->screen && info->colors[i].pict)
-    {
-	XRenderFreePicture (dpy, info->colors[i].pict);
-	info->colors[i].pict = 0;
+
+    if (info->hasSolid) {
+	/*
+	 * Free any existing entry
+	 */
+	if (info->colors[i].pict)
+	    XRenderFreePicture (dpy, info->colors[i].pict);
+	/*
+	 * Create picture
+	 */
+	info->colors[i].pict = XRenderCreateSolidFill (draw->dpy, &color->color);
+    } else {
+	if (info->colors[i].screen != draw->screen && info->colors[i].pict)
+	{
+	    XRenderFreePicture (dpy, info->colors[i].pict);
+	    info->colors[i].pict = 0;
+	}
+	/*
+	 * Create picture if necessary
+	 */
+	if (!info->colors[i].pict)
+	{
+	    Pixmap			    pix;
+	    XRenderPictureAttributes    pa;
+
+	    pix = XCreatePixmap (dpy, RootWindow (dpy, draw->screen), 1, 1,
+				 (unsigned)info->solidFormat->depth);
+	    pa.repeat = True;
+	    info->colors[i].pict = XRenderCreatePicture (draw->dpy,
+							 pix,
+							 info->solidFormat,
+							 CPRepeat, &pa);
+	    XFreePixmap (dpy, pix);
+	}
+	/*
+	 * Set to the new color
+	 */
+	info->colors[i].color = color->color;
+	info->colors[i].screen = draw->screen;
+	XRenderFillRectangle (dpy, PictOpSrc,
+			      info->colors[i].pict,
+			      &color->color, 0, 0, 1, 1);
     }
-    /*
-     * Create picture if necessary
-     */
-    if (!info->colors[i].pict)
-    {
-	Pixmap			    pix;
-        XRenderPictureAttributes    pa;
-	
-	pix = XCreatePixmap (dpy, RootWindow (dpy, draw->screen), 1, 1,
-			     info->solidFormat->depth);
-	pa.repeat = True;
-	info->colors[i].pict = XRenderCreatePicture (draw->dpy,
-						     pix,
-						     info->solidFormat,
-						     CPRepeat, &pa);
-	XFreePixmap (dpy, pix);
-    }
-    /*
-     * Set to the new color
-     */
     info->colors[i].color = color->color;
     info->colors[i].screen = draw->screen;
-    XRenderFillRectangle (dpy, PictOpSrc,
-			  info->colors[i].pict,
-			  &color->color, 0, 0, 1, 1);
+
     return info->colors[i].pict;
 }
 
@@ -393,7 +404,7 @@ _XftDrawRenderPrepare (XftDraw	*draw)
 	format = _XftDrawFormat (draw);
 	if (!format)
 	    return FcFalse;
-	
+
 	if (draw->subwindow_mode == IncludeInferiors)
 	{
 	    pa.subwindow_mode = IncludeInferiors;
@@ -456,7 +467,7 @@ _XftDrawCorePrepare (XftDraw *draw, _Xconst XftColor *color)
     XSetForeground (draw->dpy, draw->core.gc, color->pixel);
     return FcTrue;
 }
-			
+
 _X_EXPORT Picture
 XftDrawPicture (XftDraw *draw)
 {
@@ -481,7 +492,7 @@ XftDrawGlyphs (XftDraw		*draw,
     if (font->format)
     {
 	Picture	    src;
-	
+
 	if (_XftDrawRenderPrepare (draw) &&
 	    (src = XftDrawSrcPicture (draw, color)))
 	    XftGlyphRender (draw->dpy, _XftDrawOp (draw, color),
@@ -499,7 +510,7 @@ _X_EXPORT void
 XftDrawString8 (XftDraw		    *draw,
 		_Xconst XftColor    *color,
 		XftFont		    *pub,
-		int		    x, 
+		int		    x,
 		int		    y,
 		_Xconst FcChar8	    *string,
 		int		    len)
@@ -509,12 +520,12 @@ XftDrawString8 (XftDraw		    *draw,
 
     if (XftDebug () & XFT_DBG_DRAW)
 	printf ("DrawString \"%*.*s\"\n", len, len, string);
-    
+
     if (len <= NUM_LOCAL)
 	glyphs = glyphs_local;
     else
     {
-	glyphs = malloc (len * sizeof (FT_UInt));
+	glyphs = malloc ((size_t)len * sizeof (FT_UInt));
 	if (!glyphs)
 	    return;
     }
@@ -541,13 +552,13 @@ XftDrawString16 (XftDraw	    *draw,
 	glyphs = glyphs_local;
     else
     {
-	glyphs = malloc (len * sizeof (FT_UInt));
+	glyphs = malloc ((size_t)len * sizeof (FT_UInt));
 	if (!glyphs)
 	    return;
     }
     for (i = 0; i < len; i++)
 	glyphs[i] = XftCharIndex (draw->dpy, pub, string[i]);
-    
+
     XftDrawGlyphs (draw, color, pub, x, y, glyphs, len);
     if (glyphs != glyphs_local)
 	free (glyphs);
@@ -569,13 +580,13 @@ XftDrawString32 (XftDraw	    *draw,
 	glyphs = glyphs_local;
     else
     {
-	glyphs = malloc (len * sizeof (FT_UInt));
+	glyphs = malloc ((size_t)len * sizeof (FT_UInt));
 	if (!glyphs)
 	    return;
     }
     for (i = 0; i < len; i++)
 	glyphs[i] = XftCharIndex (draw->dpy, pub, string[i]);
-    
+
     XftDrawGlyphs (draw, color, pub, x, y, glyphs, len);
     if (glyphs != glyphs_local)
 	free (glyphs);
@@ -585,7 +596,7 @@ _X_EXPORT void
 XftDrawStringUtf8 (XftDraw	    *draw,
 		   _Xconst XftColor *color,
 		   XftFont	    *pub,
-		   int		    x, 
+		   int		    x,
 		   int		    y,
 		   _Xconst FcChar8  *string,
 		   int		    len)
@@ -603,14 +614,14 @@ XftDrawStringUtf8 (XftDraw	    *draw,
     {
 	if (i == size)
 	{
-	    glyphs_new = malloc (size * 2 * sizeof (FT_UInt));
+	    glyphs_new = malloc ((size_t)size * 2 * sizeof (FT_UInt));
 	    if (!glyphs_new)
 	    {
 		if (glyphs != glyphs_local)
 		    free (glyphs);
 		return;
 	    }
-	    memcpy (glyphs_new, glyphs, size * sizeof (FT_UInt));
+	    memcpy (glyphs_new, glyphs, (size_t)size * sizeof (FT_UInt));
 	    size *= 2;
 	    if (glyphs != glyphs_local)
 		free (glyphs);
@@ -648,14 +659,14 @@ XftDrawStringUtf16 (XftDraw		*draw,
     {
 	if (i == size)
 	{
-	    glyphs_new = malloc (size * 2 * sizeof (FT_UInt));
+	    glyphs_new = malloc ((size_t)size * 2 * sizeof (FT_UInt));
 	    if (!glyphs_new)
 	    {
 		if (glyphs != glyphs_local)
 		    free (glyphs);
 		return;
 	    }
-	    memcpy (glyphs_new, glyphs, size * sizeof (FT_UInt));
+	    memcpy (glyphs_new, glyphs, (size_t)size * sizeof (FT_UInt));
 	    size *= 2;
 	    if (glyphs != glyphs_local)
 		free (glyphs);
@@ -748,7 +759,7 @@ XftDrawCharSpec (XftDraw		*draw,
 	glyphs = glyphs_local;
     else
     {
-	glyphs = malloc (len * sizeof (XftGlyphSpec));
+	glyphs = malloc ((size_t)len * sizeof (XftGlyphSpec));
 	if (!glyphs)
 	    return;
     }
@@ -777,7 +788,7 @@ XftDrawCharFontSpec (XftDraw			*draw,
 	glyphs = glyphs_local;
     else
     {
-	glyphs = malloc (len * sizeof (XftGlyphFontSpec));
+	glyphs = malloc ((size_t)len * sizeof (XftGlyphFontSpec));
 	if (!glyphs)
 	    return;
     }
@@ -797,7 +808,7 @@ XftDrawCharFontSpec (XftDraw			*draw,
 _X_EXPORT void
 XftDrawRect (XftDraw		*draw,
 	     _Xconst XftColor	*color,
-	     int		x, 
+	     int		x,
 	     int		y,
 	     unsigned int	width,
 	     unsigned int	height)
@@ -809,7 +820,10 @@ XftDrawRect (XftDraw		*draw,
     }
     else if (_XftDrawCorePrepare (draw, color))
     {
-	XftRectCore (draw, color, x, y, width, height);
+	/* note: not XftRectCore() */
+	XSetForeground (draw->dpy, draw->core.gc, color->pixel);
+	XFillRectangle (draw->dpy, draw->drawable, draw->core.gc,
+			x, y, width, height);
     }
 }
 
@@ -817,16 +831,16 @@ _X_EXPORT Bool
 XftDrawSetClip (XftDraw	*draw,
 		Region	r)
 {
-    Region			n = 0;
+    Region			n = NULL;
 
     /*
      * Check for quick exits
      */
     if (!r && draw->clip_type == XftClipTypeNone)
 	return True;
-    
-    if (r && 
-	draw->clip_type == XftClipTypeRegion && 
+
+    if (r &&
+	draw->clip_type == XftClipTypeRegion &&
 	XEqualRegion (r, draw->clip.region))
     {
 	return True;
@@ -861,7 +875,7 @@ XftDrawSetClip (XftDraw	*draw,
     case XftClipTypeNone:
 	break;
     }
-    
+
     /*
      * Set the clip
      */
@@ -906,7 +920,7 @@ XftDrawSetClipRectangles (XftDraw		*draw,
 			  _Xconst XRectangle	*rects,
 			  int			n)
 {
-    XftClipRect	*new = 0;
+    XftClipRect	*new = NULL;
 
     /*
      * Check for quick exit
@@ -915,7 +929,7 @@ XftDrawSetClipRectangles (XftDraw		*draw,
 	draw->clip.rect->n == n &&
 	(n == 0 || (draw->clip.rect->xOrigin == xOrigin &&
 		    draw->clip.rect->yOrigin == yOrigin)) &&
-	!memcmp (XftClipRects (draw->clip.rect), rects, n * sizeof (XRectangle)))
+	!memcmp (XftClipRects (draw->clip.rect), rects, (size_t)n * sizeof (XRectangle)))
     {
 	return True;
     }
@@ -923,14 +937,14 @@ XftDrawSetClipRectangles (XftDraw		*draw,
     /*
      * Duplicate the region so future changes can be short circuited
      */
-    new = malloc (sizeof (XftClipRect) + n * sizeof (XRectangle));
+    new = malloc (sizeof (XftClipRect) + (size_t)n * sizeof (XRectangle));
     if (!new)
 	return False;
 
     new->n = n;
     new->xOrigin = xOrigin;
     new->yOrigin = yOrigin;
-    memcpy (XftClipRects (new), rects, n * sizeof (XRectangle));
+    memcpy (XftClipRects (new), rects, (size_t)n * sizeof (XRectangle));
 
     /*
      * Destroy existing clip
@@ -945,7 +959,7 @@ XftDrawSetClipRectangles (XftDraw		*draw,
     case XftClipTypeNone:
 	break;
     }
-    
+
     /*
      * Set the clip
      */
@@ -985,7 +999,7 @@ XftDrawSetSubwindowMode (XftDraw *draw, int mode)
 	XRenderPictureAttributes    pa;
 
 	pa.subwindow_mode = mode;
-	XRenderChangePicture (draw->dpy, draw->render.pict, 
+	XRenderChangePicture (draw->dpy, draw->render.pict,
 			      CPSubwindowMode, &pa);
     }
     if (draw->core.gc)
